@@ -52,24 +52,24 @@ public:
     {
     }
 
-    virtual bool Eq(wxVariantData& data) const wxOVERRIDE
+    virtual bool Eq(wxVariantData& data) const override
     {
         // We're only called with the objects of the same type, so the cast is
         // safe.
         return static_cast<wxBitmapBundleVariantData&>(data).m_value.IsSameAs(m_value);
     }
 
-    virtual wxString GetType() const wxOVERRIDE
+    virtual wxString GetType() const override
     {
         return wxASCII_STR("wxBitmapBundle");
     }
 
-    virtual wxClassInfo* GetValueClassInfo() wxOVERRIDE
+    virtual wxClassInfo* GetValueClassInfo() override
     {
-        return NULL;
+        return nullptr;
     }
 
-    virtual wxVariantData* Clone() const wxOVERRIDE
+    virtual wxVariantData* Clone() const override
     {
         return new wxBitmapBundleVariantData(m_value);
     }
@@ -129,12 +129,12 @@ public:
     {
     }
 
-    virtual wxSize GetDefaultSize() const wxOVERRIDE;
-    virtual wxSize GetPreferredBitmapSizeAtScale(double scale) const wxOVERRIDE;
-    virtual wxBitmap GetBitmap(const wxSize& size) wxOVERRIDE;
+    virtual wxSize GetDefaultSize() const override;
+    virtual wxSize GetPreferredBitmapSizeAtScale(double scale) const override;
+    virtual wxBitmap GetBitmap(const wxSize& size) override;
 
 protected:
-    virtual double GetNextAvailableScale(size_t& i) const wxOVERRIDE;
+    virtual double GetNextAvailableScale(size_t& i) const override;
 
 private:
     // Struct containing bitmap itself as well as a flag indicating whether we
@@ -282,7 +282,7 @@ wxBitmap wxBitmapBundleImplSet::GetBitmap(const wxSize& size)
     // We use linear search instead if binary one because it's simpler and the
     // vector size is small enough (< 10) for it not to matter in practice.
     const size_t n = m_entries.size();
-    size_t lastSmaller = 0;
+    size_t nextBigger = 0;
     for ( size_t i = 0; i < n; ++i )
     {
         const Entry& entry = m_entries[i];
@@ -298,7 +298,7 @@ wxBitmap wxBitmapBundleImplSet::GetBitmap(const wxSize& size)
 
             if ( sizeThis.x < size.x )
             {
-                lastSmaller = i;
+                nextBigger = i + 1;
                 continue;
             }
 
@@ -310,7 +310,7 @@ wxBitmap wxBitmapBundleImplSet::GetBitmap(const wxSize& size)
         {
             // Don't rescale this one, we prefer to downscale rather than
             // upscale as it results in better-looking bitmaps.
-            lastSmaller = i;
+            nextBigger = i + 1;
             continue;
         }
 
@@ -320,7 +320,7 @@ wxBitmap wxBitmapBundleImplSet::GetBitmap(const wxSize& size)
             // next bigger bitmap, so rescale it to the desired size.
             const Entry entryNew(entry, size);
 
-            m_entries.insert(m_entries.begin() + lastSmaller + 1, entryNew);
+            m_entries.insert(m_entries.begin() + nextBigger, entryNew);
 
             return entryNew.bitmap;
         }
@@ -341,7 +341,7 @@ wxBitmap wxBitmapBundleImplSet::GetBitmap(const wxSize& size)
 #ifdef __WXOSX__
 void wxBitmapBundleImplSet::OSXCreateNSImage()
 {
-    WXImage image = NULL;
+    WXImage image = nullptr;
     const size_t n = m_entries.size();
     if ( n == 1 )
     {
@@ -396,17 +396,17 @@ wxBitmapBundle::wxBitmapBundle(wxBitmapBundleImpl* impl)
 }
 
 wxBitmapBundle::wxBitmapBundle(const wxBitmap& bitmap)
-    : m_impl(bitmap.IsOk() ? new wxBitmapBundleImplSet(bitmap) : NULL)
+    : m_impl(bitmap.IsOk() ? new wxBitmapBundleImplSet(bitmap) : nullptr)
 {
 }
 
 wxBitmapBundle::wxBitmapBundle(const wxIcon& icon)
-    : m_impl(icon.IsOk() ? new wxBitmapBundleImplSet(wxBitmap(icon)) : NULL)
+    : m_impl(icon.IsOk() ? new wxBitmapBundleImplSet(wxBitmap(icon)) : nullptr)
 {
 }
 
 wxBitmapBundle::wxBitmapBundle(const wxImage& image)
-    : m_impl(image.IsOk() ? new wxBitmapBundleImplSet(wxBitmap(image)) : NULL)
+    : m_impl(image.IsOk() ? new wxBitmapBundleImplSet(wxBitmap(image)) : nullptr)
 {
 }
 
@@ -433,7 +433,7 @@ wxBitmapBundle::~wxBitmapBundle()
 
 void wxBitmapBundle::Clear()
 {
-    m_impl.reset(NULL);
+    m_impl.reset(nullptr);
 }
 
 /* static */
@@ -515,7 +515,6 @@ wxBitmapBundle wxBitmapBundle::FromFiles(const wxString& path, const wxString& f
     wxVector<wxBitmap> bitmaps;
 
     wxFileName fn(path, filename, extension);
-    wxString ext = extension.Lower();
 
     for ( int dpiFactor = 1 ; dpiFactor <= 2 ; ++dpiFactor)
     {
@@ -649,7 +648,7 @@ void RecordSizePref(SizePrefs& prefs, const wxSize& size)
 
 /* static */
 wxSize
-wxBitmapBundle::GetConsensusSizeFor(wxWindow* win,
+wxBitmapBundle::GetConsensusSizeFor(const wxWindow* win,
                                     const wxVector<wxBitmapBundle>& bundles)
 {
     return GetConsensusSizeFor(win->GetDPIScaleFactor(), bundles);
@@ -699,17 +698,13 @@ wxBitmapBundle::GetConsensusSizeFor(double scale,
 
 /* static */
 wxImageList*
-wxBitmapBundle::CreateImageList(wxWindow* win,
+wxBitmapBundle::CreateImageList(const wxWindow* win,
                                 const wxVector<wxBitmapBundle>& bundles)
 {
-    wxCHECK_MSG( win, NULL, "must have a valid window" );
-    wxCHECK_MSG( !bundles.empty(), NULL, "should have some images" );
+    wxCHECK_MSG( win, nullptr, "must have a valid window" );
+    wxCHECK_MSG( !bundles.empty(), nullptr, "should have some images" );
 
-    wxSize size = GetConsensusSizeFor(win, bundles);
-
-    // wxImageList wants the logical size for the platforms where logical and
-    // physical pixels are different.
-    size /= win->GetContentScaleFactor();
+    const wxSize size = GetConsensusSizeFor(win, bundles);
 
     wxImageList* const iml = new wxImageList(size.x, size.y);
 
@@ -849,15 +844,13 @@ size_t wxBitmapBundleImpl::GetIndexToUpscale(const wxSize& size) const
     const wxSize sizeDef = GetDefaultSize();
     for ( size_t i = 0;; )
     {
-        // Save it before it's updated by GetNextAvailableScale().
-        size_t indexPrev = i;
-
         const double scaleThis = GetNextAvailableScale(i);
         if ( scaleThis == 0.0 )
             break;
 
-        // Only update it now, knowing that this index could have been used.
-        indexLast = indexPrev;
+        // Only update it now, knowing that the index was changed by
+        // GetNextAvailableScale() to be one beyond the actually used one.
+        indexLast = i - 1;
 
         const double scale = size.y / (sizeDef.y*scaleThis);
         if (wxRound(scale) == scale)
