@@ -2,7 +2,6 @@
 // Name:        src/common/paper.cpp
 // Purpose:     Paper size classes
 // Author:      Julian Smart
-// Modified by:
 // Created:     04/01/98
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
@@ -16,10 +15,8 @@
 
 #ifndef WX_PRECOMP
     #if defined(__WXMSW__)
-        #include "wx/msw/wrapcdlg.h"
+        #include "wx/msw/wrapwin.h" // for DMPAPER_XXX constants
     #endif // MSW
-    #include "wx/utils.h"
-    #include "wx/settings.h"
     #include "wx/intl.h"
     #include "wx/module.h"
 #endif
@@ -29,15 +26,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#ifdef __WXMSW__
-    #ifndef __WIN32__
-        #include <print.h>
-    #endif
-#endif
- // End __WXMSW__
-
 wxIMPLEMENT_DYNAMIC_CLASS(wxPrintPaperType, wxObject);
-// wxIMPLEMENT_DYNAMIC_CLASS(wxPrintPaperDatabase, wxList);
 
 /*
  * Paper size database for all platforms
@@ -70,22 +59,11 @@ wxSize wxPrintPaperType::GetSizeDeviceUnits() const
  * Print paper database for PostScript
  */
 
-WX_DECLARE_LIST(wxPrintPaperType, wxPrintPaperTypeList);
-#include "wx/listimpl.cpp"
-WX_DEFINE_LIST(wxPrintPaperTypeList)
+wxPrintPaperDatabase* wxThePrintPaperDatabase = nullptr;
 
-wxPrintPaperDatabase* wxThePrintPaperDatabase = NULL;
+wxPrintPaperDatabase::wxPrintPaperDatabase() = default;
 
-wxPrintPaperDatabase::wxPrintPaperDatabase()
-{
-    m_map = new wxStringToPrintPaperTypeHashMap;
-    m_list = new wxPrintPaperTypeList;
-}
-
-wxPrintPaperDatabase::~wxPrintPaperDatabase()
-{
-    ClearDatabase();
-}
+wxPrintPaperDatabase::~wxPrintPaperDatabase() = default;
 
 void wxPrintPaperDatabase::CreateDatabase()
 {
@@ -217,60 +195,50 @@ void wxPrintPaperDatabase::CreateDatabase()
 
 void wxPrintPaperDatabase::ClearDatabase()
 {
-    delete m_list;
-    WX_CLEAR_HASH_MAP(wxStringToPrintPaperTypeHashMap, *m_map);
-    delete m_map;
+    m_paperTypes.clear();
 }
 
 void wxPrintPaperDatabase::AddPaperType(wxPaperSize paperId, const wxString& name, int w, int h)
 {
-    wxPrintPaperType* tmp = new wxPrintPaperType(paperId, 0, name, w, h);
-    (*m_map)[name] = tmp;
-    m_list->push_back(tmp);
+    m_paperTypes.emplace_back(paperId, 0, name, w, h);
 }
 
 void wxPrintPaperDatabase::AddPaperType(wxPaperSize paperId, int platformId, const wxString& name, int w, int h)
 {
-    wxPrintPaperType* tmp = new wxPrintPaperType(paperId, platformId, name, w, h);
-    (*m_map)[name] = tmp;
-    m_list->push_back(tmp);
+    m_paperTypes.emplace_back(paperId, platformId, name, w, h);
 }
 
 wxPrintPaperType *wxPrintPaperDatabase::FindPaperType(const wxString& name) const
 {
-    wxStringToPrintPaperTypeHashMap::iterator it = m_map->find(name);
-    if (it != m_map->end())
-        return it->second;
-    else
-        return NULL;
+    for (const auto& paperType : m_paperTypes)
+    {
+        if (paperType.GetName() == name)
+            return AsNonConstPtr(paperType);
+    }
+
+    return nullptr;
 }
 
 wxPrintPaperType *wxPrintPaperDatabase::FindPaperType(wxPaperSize id) const
 {
-    typedef wxStringToPrintPaperTypeHashMap::iterator iterator;
-
-    for (iterator it = m_map->begin(), en = m_map->end(); it != en; ++it)
+    for (const auto& paperType : m_paperTypes)
     {
-        wxPrintPaperType* paperType = it->second;
-        if (paperType->GetId() == id)
-            return paperType;
+        if (paperType.GetId() == id)
+            return AsNonConstPtr(paperType);
     }
 
-    return NULL;
+    return nullptr;
 }
 
 wxPrintPaperType *wxPrintPaperDatabase::FindPaperTypeByPlatformId(int id) const
 {
-    typedef wxStringToPrintPaperTypeHashMap::iterator iterator;
-
-    for (iterator it = m_map->begin(), en = m_map->end(); it != en; ++it)
+    for (const auto& paperType : m_paperTypes)
     {
-        wxPrintPaperType* paperType = it->second;
-        if (paperType->GetPlatformId() == id)
-            return paperType;
+        if (paperType.GetPlatformId() == id)
+            return AsNonConstPtr(paperType);
     }
 
-    return NULL;
+    return nullptr;
 }
 
 wxPrintPaperType *wxPrintPaperDatabase::FindPaperType(const wxSize& sz) const
@@ -279,15 +247,14 @@ wxPrintPaperType *wxPrintPaperDatabase::FindPaperType(const wxSize& sz) const
     // are likely to be taken into account first. This fixes problems with,
     // for example, Letter reverting to A4 in the page setup dialog because
     // it was wrongly translated to Note.
-    for ( size_t i = 0; i < GetCount(); i++ )
+    for (const auto& paperType : m_paperTypes)
     {
-        wxPrintPaperType* const paperType = Item(i);
-        const wxSize paperSize = paperType->GetSize() ;
+        const wxSize paperSize = paperType.GetSize() ;
         if ( abs(paperSize.x - sz.x) < 10 && abs(paperSize.y - sz.y) < 10 )
-            return paperType;
+            return AsNonConstPtr(paperType);
     }
 
-    return NULL;
+    return nullptr;
 }
 
 // Convert name to size id
@@ -333,12 +300,15 @@ wxPaperSize wxPrintPaperDatabase::GetSize(const wxSize& size) const
 // QUICK and DIRTY
 size_t wxPrintPaperDatabase::GetCount() const
 {
-    return m_list->GetCount();
+    return m_paperTypes.size();
 }
 
 wxPrintPaperType* wxPrintPaperDatabase::Item(size_t index) const
 {
-    return m_list->Item(index)->GetData();
+    if (index >= m_paperTypes.size())
+        return nullptr;
+
+    return AsNonConstPtr(m_paperTypes[index]);
 }
 
 // A module to allow initialization/cleanup of print paper
@@ -349,8 +319,8 @@ class WXDLLEXPORT wxPrintPaperModule: public wxModule
     wxDECLARE_DYNAMIC_CLASS(wxPrintPaperModule);
 public:
     wxPrintPaperModule() {}
-    bool OnInit() wxOVERRIDE;
-    void OnExit() wxOVERRIDE;
+    bool OnInit() override;
+    void OnExit() override;
 };
 
 wxIMPLEMENT_DYNAMIC_CLASS(wxPrintPaperModule, wxModule);
