@@ -16,7 +16,6 @@
 #ifndef WX_PRECOMP
     #include "wx/intl.h"
     #include "wx/log.h"
-    #include "wx/utils.h"
     #include "wx/msgdlg.h"
     #include "wx/bitmap.h"
 #endif
@@ -40,19 +39,17 @@
 //     class which explains it but it still would be nice to do something
 //     about this one day
 
-class wxGtkNotebookPage: public wxObject
+class wxGtkNotebookPage
 {
 public:
     GtkWidget* m_box;
     GtkWidget* m_label;
     GtkWidget* m_image;
     int m_imageIndex;
+    wxString m_text;
 };
 
 
-#include "wx/listimpl.cpp"
-WX_DEFINE_LIST(wxGtkNotebookPagesList)
-
 extern "C" {
 static void event_after(GtkNotebook*, GdkEvent*, wxNotebook*);
 }
@@ -209,7 +206,7 @@ bool wxNotebook::Create(wxWindow *parent, wxWindowID id,
 
 int wxNotebook::GetSelection() const
 {
-    wxCHECK_MSG( m_widget != NULL, wxNOT_FOUND, wxT("invalid notebook") );
+    wxCHECK_MSG( m_widget != nullptr, wxNOT_FOUND, wxT("invalid notebook") );
 
     return gtk_notebook_get_current_page( GTK_NOTEBOOK(m_widget) );
 }
@@ -218,8 +215,7 @@ wxString wxNotebook::GetPageText( size_t page ) const
 {
     wxCHECK_MSG(page < GetPageCount(), wxEmptyString, "invalid notebook index");
 
-    GtkLabel* label = GTK_LABEL(GetNotebookPage(page)->m_label);
-    return wxGTK_CONV_BACK(gtk_label_get_text(label));
+    return GetNotebookPage(page)->m_text;
 }
 
 int wxNotebook::GetPageImage( size_t page ) const
@@ -231,7 +227,7 @@ int wxNotebook::GetPageImage( size_t page ) const
 
 wxGtkNotebookPage* wxNotebook::GetNotebookPage( int page ) const
 {
-    return m_pagesData.Item(page)->GetData();
+    return const_cast<wxGtkNotebookPage*>(&m_pagesData.at(page));
 }
 
 int wxNotebook::DoSetSelection( size_t page, int flags )
@@ -272,8 +268,11 @@ bool wxNotebook::SetPageText( size_t page, const wxString &text )
 {
     wxCHECK_MSG(page < GetPageCount(), false, "invalid notebook index");
 
-    GtkLabel* label = GTK_LABEL(GetNotebookPage(page)->m_label);
-    gtk_label_set_text(label, wxGTK_CONV(text));
+    wxGtkNotebookPage* const pageData = GetNotebookPage(page);
+    pageData->m_text = text;
+
+    GtkLabel* label = GTK_LABEL(pageData->m_label);
+    gtk_label_set_text(label, RemoveMnemonics(text).utf8_str());
 
     return true;
 }
@@ -287,7 +286,7 @@ bool wxNotebook::SetPageImage( size_t page, int image )
     const wxBitmapBundle bundle = GetBitmapBundle(image);
     if ( bundle.IsOk() )
     {
-        if (pageData->m_image == NULL)
+        if (pageData->m_image == nullptr)
         {
             pageData->m_image = wxGtkImage::New();
             gtk_widget_show(pageData->m_image);
@@ -299,7 +298,7 @@ bool wxNotebook::SetPageImage( size_t page, int image )
     else if (pageData->m_image)
     {
         gtk_container_remove(GTK_CONTAINER(pageData->m_box), pageData->m_image);
-        pageData->m_image = NULL;
+        pageData->m_image = nullptr;
     }
     pageData->m_imageIndex = image;
 
@@ -314,27 +313,27 @@ wxSize wxNotebook::CalcSizeFromPage(const wxSize& sizePage) const
     for ( size_t n = 0; n < pageCount; n++ )
     {
         GtkRequisition req;
-        gtk_widget_get_preferred_size(GetNotebookPage(n)->m_box, NULL, &req);
+        gtk_widget_get_preferred_size(GetNotebookPage(n)->m_box, nullptr, &req);
         sizeTabMax.IncTo(wxSize(req.width, req.height));
     }
 
     wxSize sizeFull(sizePage);
 #ifdef __WXGTK3__
     GtkBorder b;
-    if (gtk_check_version(3,20,0) == NULL)
+    if (gtk_check_version(3,20,0) == nullptr)
     {
         wxGtkStyleContext sc;
-        sc.Add(GTK_TYPE_NOTEBOOK, "notebook", "notebook", "frame", NULL);
+        sc.Add(GTK_TYPE_NOTEBOOK, "notebook", "notebook", "frame", nullptr);
         gtk_style_context_get_border(sc, GTK_STATE_FLAG_NORMAL, &b);
         sizeFull.IncBy(b.left + b.right, b.top + b.bottom);
 
-        sc.Add(G_TYPE_NONE, "header", IsVertical() ? "top" : "left", NULL);
-        sc.Add(G_TYPE_NONE, "tabs", NULL);
-        sc.Add(G_TYPE_NONE, "tab", NULL);
+        sc.Add(G_TYPE_NONE, "header", IsVertical() ? "top" : "left", nullptr);
+        sc.Add(G_TYPE_NONE, "tabs", nullptr);
+        sc.Add(G_TYPE_NONE, "tab", nullptr);
 
         wxSize tabMin;
         gtk_style_context_get(sc, GTK_STATE_FLAG_NORMAL,
-            "min-width", &tabMin.x, "min-height", &tabMin.y, NULL);
+            "min-width", &tabMin.x, "min-height", &tabMin.y, nullptr);
         sizeTabMax.IncTo(tabMin);
 
         gtk_style_context_get_margin(sc, GTK_STATE_FLAG_NORMAL, &b);
@@ -352,7 +351,11 @@ wxSize wxNotebook::CalcSizeFromPage(const wxSize& sizePage) const
 
         GtkStyleContext* sc = gtk_widget_get_style_context(m_widget);
         gtk_style_context_save(sc);
+
+        wxGCC_WARNING_SUPPRESS(deprecated-declarations)
         gtk_style_context_add_region(sc, "tab", GtkRegionFlags(0));
+        wxGCC_WARNING_RESTORE(deprecated-declarations)
+
         gtk_style_context_add_class(sc, "top");
         gtk_style_context_get_padding(sc, GTK_STATE_FLAG_NORMAL, &b);
         sizeTabMax.IncBy(b.left + b.right, b.top + b.bottom);
@@ -367,8 +370,8 @@ wxSize wxNotebook::CalcSizeFromPage(const wxSize& sizePage) const
 
     int tab_hborder, tab_vborder, focus_width;
     g_object_get(G_OBJECT(m_widget),
-        "tab-hborder", &tab_hborder, "tab-vborder", &tab_vborder, NULL);
-    gtk_widget_style_get(m_widget, "focus-line-width", &focus_width, NULL);
+        "tab-hborder", &tab_hborder, "tab-vborder", &tab_vborder, nullptr);
+    gtk_widget_style_get(m_widget, "focus-line-width", &focus_width, nullptr);
     sizeTabMax.x += 2 * (tab_hborder + focus_width + m_widget->style->xthickness);
     sizeTabMax.y += 2 * (tab_vborder + focus_width + m_widget->style->ythickness);
 
@@ -386,7 +389,7 @@ wxSize wxNotebook::CalcSizeFromPage(const wxSize& sizePage) const
 
 void wxNotebook::SetPadding( const wxSize &padding )
 {
-    wxCHECK_RET( m_widget != NULL, wxT("invalid notebook") );
+    wxCHECK_RET( m_widget != nullptr, wxT("invalid notebook") );
 
     m_padding = padding.GetWidth();
 
@@ -425,7 +428,7 @@ wxNotebookPage *wxNotebook::DoRemovePage( size_t page )
     // of sync during the PAGE_CHANGING/PAGE_CHANGED events.
     wxNotebookPage *client = GetPage(page);
     if ( !client )
-        return NULL;
+        return nullptr;
 
     // Suppress bogus assertion failures happening deep inside ATK (used by
     // GTK) that can't be avoided in any other way, see #22176.
@@ -441,9 +444,7 @@ wxNotebookPage *wxNotebook::DoRemovePage( size_t page )
     wxASSERT_MSG(GetPage(page) == client, wxT("pages changed during delete"));
     wxNotebookBase::DoRemovePage(page);
 
-    wxGtkNotebookPage* p = GetNotebookPage(page);
-    m_pagesData.DeleteObject(p);
-    delete p;
+    m_pagesData.erase(m_pagesData.begin() + page);
 
     return client;
 }
@@ -454,7 +455,7 @@ bool wxNotebook::InsertPage( size_t position,
                              bool select,
                              int imageId )
 {
-    wxCHECK_MSG( m_widget != NULL, false, wxT("invalid notebook") );
+    wxCHECK_MSG( m_widget != nullptr, false, wxT("invalid notebook") );
 
     wxCHECK_MSG( win->GetParent() == this, false,
                wxT("Can't add a page whose parent is not the notebook!") );
@@ -471,10 +472,10 @@ bool wxNotebook::InsertPage( size_t position,
 
     GtkNotebook *notebook = GTK_NOTEBOOK(m_widget);
 
-    wxGtkNotebookPage* pageData = new wxGtkNotebookPage;
-
     m_pages.insert(m_pages.begin() + position, win);
-    m_pagesData.Insert(position, pageData);
+    m_pagesData.insert(m_pagesData.begin() + position, wxGtkNotebookPage());
+
+    wxGtkNotebookPage* const pageData = &m_pagesData[position];
 
     // set the label image and text
     // this must be done before adding the page, as GetPageText
@@ -498,11 +499,14 @@ bool wxNotebook::InsertPage( size_t position,
     }
     else
     {
-        pageData->m_image = NULL;
+        pageData->m_image = nullptr;
     }
 
-    /* set the label text */
-    pageData->m_label = gtk_label_new(wxGTK_CONV(wxStripMenuCodes(text)));
+    // Set the label text: we don't support mnemonics here, but we still need
+    // to strip them if there are any. Also store the original text to be able
+    // to return it from GetPageText() later.
+    pageData->m_text = text;
+    pageData->m_label = gtk_label_new(RemoveMnemonics(text).utf8_str());
 
     if (m_windowStyle & wxBK_LEFT)
         gtk_label_set_angle(GTK_LABEL(pageData->m_label), 90);
@@ -522,7 +526,7 @@ bool wxNotebook::InsertPage( size_t position,
 
     /* apply current style */
 #ifdef __WXGTK3__
-    GTKApplyStyle(pageData->m_label, NULL);
+    GTKApplyStyle(pageData->m_label, nullptr);
 #else
     GtkRcStyle *style = GTKCreateWidgetStyle();
     if ( style )
@@ -659,7 +663,7 @@ GdkWindow *wxNotebook::GTKGetWindow(wxArrayGdkWindows& windows) const
     windows.push_back(GTK_NOTEBOOK(m_widget)->event_window);
 #endif
 
-    return NULL;
+    return nullptr;
 }
 
 // static
